--[[
    Author: KepocuH

    This is a space scanning plugin, which looks asynchronously for an objects, those names may
    include elements of search_list as their parts. The scan continues until the object is found
    or scan turned off manually. You may add the search patterns to the search_list by /scan_add
    command and remove them  by /scan_del. You may toggle the scan by issuing the /scan_toggle
    command.

    scan_add and scan_del are accepting multipli arguments at once

    /scan_add arthemis nanite
    /scan_del orun scrap

    It is practical that you bind the toggle to some button like:
    -----
    /bind F2 scan_toggle
    -----


    The origin of sound10.ogg and sound12.ogg are the  SoundAlertSuite plugin. 

--]]

require(io);

spaceScan = spaceScan or  {};
utils = utils or {};

local properties = {
    search_list={},
    timeout=50,
    rescan=5000,
    rescan_delta=500
}

local persistent_properties="spaceScan.";

local __CMD="RadarNext";

local __dosearch=false;
local __SearchTimer=Timer();
local __RescanTimer=Timer();
local __SL_Iterator=1;
local __SoundNameLookup="sound10.ogg";
local __SoundNameFound="sound12.ogg";
local __MasterVolume=tonumber(gkini.ReadString("SAS", "master_volume", '1.0'));

local __search_sound = gksound.GKLoadSound{soundname=__SoundNameLookup,filename="plugins/spaceScan/sounds/"..__SoundNameLookup};
local __found_sound = gksound.GKLoadSound{soundname=__SoundNameFound,filename="plugins/spaceScan/sounds/"..__SoundNameFound};

function lookup()
   local name="XUBADOOO"

   __SearchTimer:Kill();

   gkinterface.GKProcessCommand(__CMD);
   name = GetTargetInfo()
   if((name ~=nil) and (name ~= "XUBADOOO"))
   then
        local tmpstr = name:lower()

        if(__SL_Iterator > table.maxn(properties.search_list))
        then
            __SL_Iterator=1;
        elseif(__SL_Iterator == nil)
        then
            __SL_Iterator=1;
        end

        if(tmpstr:find(properties.search_list[__SL_Iterator]:lower()) and (not tmpstr:match(" nanite ")))
        then
            gksound.GKPlaySound(__SoundNameFound, __MasterVolume);
            generalprint("Found: "..tmpstr)
            __dosearch=false;
        else
            __SearchTimer:SetTimeout(properties.timeout,reset_timer);
        end
   else
        __SL_Iterator=__SL_Iterator + 1;
        if(__CMD == "RadarNext")
        then
            __CMD="RadarPrev";
        else
            __CMD="RadarNext";
        end
        __SearchTimer:SetTimeout(properties.timeout,reset_timer);
   end
end

function reset_timer()
    __SearchTimer:Kill();
    if(__dosearch)
    then
        lookup();
        __SearchTimer:SetTimeout(properties.timeout,reset_timer);
    end
end

function search_heart_bit()
    if(__dosearch)
    then
        gksound.GKPlaySound(__SoundNameLookup, __MasterVolume);
        radar.SetRadarSelection(0,0)
        __RescanTimer:SetTimeout(properties.rescan, search_heart_bit);
    else
        __RescanTimer:Kill()
    end
end

function spaceScan:toggle(...)
    gksound.GKPlaySound(__SoundNameLookup, __MasterVolume);
    radar.SetRadarSelection(0,0)

    if(__dosearch)
    then
        __dosearch=false;
    else
        __dosearch=true;
    end

    search_heart_bit()
    reset_timer();
end

function spaceScan:add(data)
    if(data ~= nil)
    then
        for i=1,table.maxn(data)
        do
            table.insert(properties.search_list,data[i]:lower());
        end

        local export_list=""
        for i=1,table.maxn(properties.search_list)
        do
            if(export_list == "")
            then
                export_list=properties.search_list[i];
            else
                export_list=export_list..":"..properties.search_list[i]
            end
        end
        gkini.WriteString(persistent_properties,"search_list",export_list);
    end
end

function spaceScan:del(data)
    if(data ~= nil)
    then
        for j=1,table.maxn(data)
        do
            for i=1,table.maxn(properties.search_list)
            do
                if(properties.search_list[i]==data[j]:lower())
                then
                    table.remove(properties.search_list,i);
                    local export_list=""
                    for m=1,table.maxn(properties.search_list)
                    do
                        if(export_list == "")
                        then
                            export_list=properties.search_list[m]
                        else
                            export_list=export_list..":"..properties.search_list[m]
                        end
                    end
                    gkini.WriteString(persistent_properties,"search_list",export_list);
                    break
                end
            end
        end
    end
end

function spaceScan:list()
    generalprint("spaceScan search list includes:");
    for i=1,table.maxn(properties.search_list)
    do
        generalprint(properties.search_list[i]);
    end
end

function spaceScan:set_rescan_timer(data)
    if(data ~= nil)
    then
        local rtm=tonumber(data[1])
        if(rtm ~= nil and rtm > 4999)
        then
            properties.rescan=rtm;
            generalprint("Scan heartbit is set to "..properties.rescan.."ms");
            gkini.WriteString(persistent_properties,"rescan",properties.rescan);
        else
            generalprint("==============================");
            generalprint("try /set_rescan_timer number");
            generalprint("where the number is larger then 4999 and represents miliseconds until next radar reset");
            generalprint("==============================");
        end
    end
end

function spaceScan:set_rescan_delta(data)
    if(data ~= nil)
    then
        local delta=tonumber(data[1]);

        if(delta ~= nil)
        then
            if(delta > 0)
            then
                properties.rescan_delta=delta
                gkini.WriteString(persistent_properties,"rescan_delta",properties.rescan_delta);
                generalprint("The rescan timer adjustment delta is set to "..properties.rescan_delta.."ms");
            end
        end
    end
end

function spaceScan:set_rescan_timer_inc()
    properties.rescan=properties.rescan + properties.rescan_delta;
    gkini.WriteString(persistent_properties,"rescan",properties.rescan);
    generalprint("Rescan heart-bit timer is set to "..properties.rescan.."ms");
end

function spaceScan:set_rescan_timer_dec()
    local nrescan=(properties.rescan - properties.rescan_delta)
    if( nrescan > 4999)
    then
        properties.rescan=nrescan
        gkini.WriteString(persistent_properties,"rescan",properties.rescan);
        generalprint("Rescan heart-bit timer is set to "..properties.rescan.."ms");
    else
        generalprint("Can't set rescan timer to lower value");
    end
end

function spaceScan:init()
    persistent_properties="spaceScan."..GetPlayerName();

    local sstr=gkini.ReadString(persistent_properties,"search_list","xit:hive:guard:synth:grade:neural");

    for word in sstr:gmatch("[^:]+")
    do
        table.insert(properties.search_list,word)
    end

    local tm=tonumber(gkini.ReadString(persistent_properties,"timeout","50"));
    local rs=tonumber(gkini.ReadString(persistent_properties,"rescan","20000"));
    local d=tonumber(gkini.ReadString(persistent_properties,"delta","500"));

    properties.timeout=tm;
    properties.rescan=rs;
    properties.rescan_delta=d;

    gkini.WriteString(persistent_properties,"timeout",properties.timeout);
    gkini.WriteString(persistent_properties,"rescan",properties.rescan);
    gkini.WriteString(persistent_properties,"rescan_delta",properties.rescan_delta);
    
    RegisterUserCommand("scan_toggle",spaceScan.toggle)
    RegisterUserCommand("scan_add",spaceScan.add)
    RegisterUserCommand("scan_del",spaceScan.del)
    RegisterUserCommand("scan_list",spaceScan.list)
    RegisterUserCommand("set_rescan_timer",spaceScan.set_rescan_timer)
    RegisterUserCommand("rescan_timer_delta",spaceScan.set_rescan_delta)
    RegisterUserCommand("rescan_timer_inc",spaceScan.set_rescan_timer_inc)
    RegisterUserCommand("rescan_timer_dec",spaceScan.set_rescan_timer_dec)
end

RegisterEvent(spaceScan.init, "PLAYER_ENTERED_GAME")
